#include "stdafx.h"
#include "TaskMil.h"

/*
Mil dialogue handler 

Contents : 
	CVwMil- Options dialog 	
	VwMilDlgProc - message pump for options dialog 
	OpenVwMil( HWND hDlg, CData * ) - external entry point to open task options dialog 
*/

//-- Global data 
char * g_szTitleMil = "MIL rates";

//-----------------------------------------------------------------------------
// dialog for options edit 
class CVwMil : public CDialog { 
	private : 
		bool m_bStimSetup;	
		bool m_bNoUpdate;
			
		//-- Data source 
		CDlgCombo m_cmbDataSource;
		CDlgEdit m_edtDataChannel;
		CDlgCheck m_selWholeFile;
		CDlgCheck m_selList;
		CDlgCheck m_selCurSel;
		CDlgEdit m_edtDeadtime;
		CDlgEdit m_edtChannels;

		//-- stimulii 
		CDlgList m_lstStimulii;
		CDlgCheck m_selStimValue;
		CDlgCheck m_selStimChannel;
		CDlgEdit m_edtStimValue;
		CDlgEdit m_edtStimChannel;

		//-- mode evaluate / optimize 
		CDlgCheck m_selModeEvaluate;
		CDlgCheck m_selModeOptimize;
		CDlgEdit m_edtOptIter;
		CDlgEdit m_edtOptLLConv;
		CDlgEdit m_edtOptGradConv;
		CDlgCheck m_selOptNumericDerivatives;
		CDlgEdit m_edtOptK0Step;
		CDlgEdit m_edtOptK1Step;
		
			
	public : 		
		COptionsMil * m_pConfig;		// ptr to structure to store control values when done

		CVwMil() { 
			m_iID = IDD_TASK_MIL;
			m_pConfig = NULL;
			m_bStimSetup=false;
			m_bNoUpdate=true;
			}
	
		~CVwMil() { 
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			m_cmbDataSource.SetHandle( GetDlgItem( m_hDlg, IDC_CMB_DATASOURCE), "Idealized data source");
			m_edtDataChannel.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_DATACHANNEL), "Channel in data file which contains idealized event list");
			m_selWholeFile.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_WHOLEFILE), "Use entire file");
			m_selList.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_LIST), "Use selection list");
			m_selCurSel.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_CURSEL), "Use highlighted data selection");
			m_edtDeadtime.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_DEADTIME), "Time, in seconds, for minimum event length.");
			m_edtChannels.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_CHANNELS), "Number of Ion channels represented in the data");

			m_lstStimulii.SetHandle( GetDlgItem( m_hDlg, IDC_LST_STIMULII), "Stimulus names detected in the model.");
			m_selStimValue.SetHandle( GetDlgItem( m_hDlg, IDC_OPTSTIMTYPEVALUE), "Stimulus is a fixed value");
			m_selStimChannel.SetHandle( GetDlgItem( m_hDlg, IDC_OPTSTIMTYPECHANNEL), "Stimulus is a channel in the data file");
			m_edtStimValue.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_STIMVALUE), "Fixed value to use for stimulus");
			m_edtStimChannel.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_STIMCHANNEL), "Data channel in data source which represents the stimulus.");

			m_edtOptIter.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_OPTITER), "Exit if maximum iterations reached");
			m_edtOptLLConv.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_OPTLLCONV), "Exit if Log-likelihood increases less than this amount.");
			m_edtOptGradConv.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_OPTGRADCONV), "Exit if gradient goes below this amount.");
			m_selModeEvaluate.SetHandle( GetDlgItem( m_hDlg, IDC_SELMODEEVALUATE), "Calculate log-likelihood for selected model and data.");
			m_selModeOptimize.SetHandle( GetDlgItem( m_hDlg, IDC_SELMODEOPTIMIZE), "Optimize model rates.");
			m_selOptNumericDerivatives.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_OPTNUMERICALDERIVATIVES), "Use numerically computed derivatives to modify rate parameters.");
			m_edtOptK0Step.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_OPTK0STEP), "Size of initial step used to compute derivative dLL/dK0(i)");
			m_edtOptK1Step.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_OPTK1STEP), "Size of initial step used to compute derivative dLL/dK1(i)");
			}

		//-- 
		void FillDataSourceList( int iSelect ) { 
			//--
			m_cmbDataSource.ClearContents();

			//-- add an item for each open data view
			int iSels=0, iWin = WindowListFindID( IDD_DATAVIEW, 0 );
			while( iWin != -1 ) { 
				HWND hwndData = WindowListHandle( iWin );
				char * pchFile = (char *)(((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 1, NULL ));
				m_cmbDataSource.AddString( pchFile );
				++iSels;
				iWin = WindowListFindID( IDD_DATAVIEW, iWin+1 );
				}
				
			if( iSelect>=0 && iSelect<iSels ) 
				m_cmbDataSource.SetCurSel( iSelect );
			}


		//-- 
		void StimulusText( int iStim, int iPos ) { 
			COptionsMilStimulus * pStim = m_pConfig->Stimulus(iStim);
			char buf[256]; 
			if( pStim->m_bValue ) 
				sprintf( buf, "%s = %f", pStim->m_cName.ConstBuffer(), pStim->m_dValue );
			else
				sprintf( buf, "%s = Channel %d", pStim->m_cName.ConstBuffer(), pStim->m_iChannel );
			if( iPos==-1 ) 
				m_lstStimulii.AddString( buf );					
			else {
				m_lstStimulii.ReplaceString( iPos, buf );		
				m_lstStimulii.SetCurSel( iPos );
				}
			}

		//--
		void SetupStimulii( int iSelect ) { 
			m_lstStimulii.ClearContents();
			int iStim = m_pConfig->m_aStim.Count();
			m_bStimSetup=false;
			
			//-- Show/hide stimulus controls if any stimulii exist
			m_lstStimulii.SetVisible( iStim>0 );
			m_selStimValue.SetVisible( iStim>0 );
			m_selStimChannel.SetVisible( iStim>0 );
			m_edtStimValue.SetVisible( iStim>0 );
			m_edtStimChannel.SetVisible( iStim>0 );
			
			//-- Fill in stings 
			for( int i=0; i<iStim; ++i )
				StimulusText( i, -1 );

			//-- 
			if( iStim>0 ) 			
				m_bStimSetup=true;
				
			//-- set current select
			if( iSelect>=0 && iSelect<iStim ) {
				m_lstStimulii.SetCurSel( iSelect );
				OnStimulusSelectionChanged( );
				}
			}

		//-- When stimulus selection changes, update edit controls 
		void OnStimulusSelectionChanged( ) { 
			if( ! m_bStimSetup ) 
				return;
			m_bNoUpdate=true;				
			int iSel = m_lstStimulii.GetCurSel();
			COptionsMilStimulus * pStim = m_pConfig->Stimulus(iSel);			
			m_selStimChannel.SetCheck( ! pStim->m_bValue );
			m_selStimValue.SetCheck( pStim->m_bValue );

			m_edtStimValue.SetTextDouble( pStim->m_dValue );
			m_edtStimChannel.SetTextInt( pStim->m_iChannel );
			m_bNoUpdate=false;				
			}

		//-----------------------------------------------------------------------------
		void OnStimulusUpdate() { 
			if( m_bNoUpdate ) 
				return;
			m_bNoUpdate=true;
			int iStims = m_pConfig->m_aStim.Count();
			if( iStims==0 ) 
				return;
			int iStim = m_lstStimulii.GetCurSel();
			if( iStim < 0 ) 
				return;
			COptionsMilStimulus * pStim = m_pConfig->Stimulus(iStim);
			pStim->m_bValue = m_selStimValue.GetCheck();
			pStim->m_dValue = m_edtStimValue.GetTextDouble();
			pStim->m_iChannel = m_edtStimChannel.GetTextInt();
			StimulusText( iStim, iStim );
			m_bNoUpdate=false;
			}
			

		//-- 
		void SetControlValues( ) { 
			FillDataSourceList(0);
			m_cmbDataSource.SetCurSel( m_pConfig->m_cDataName.ConstBuffer() );

			m_selWholeFile.SetCheck( m_pConfig->m_iDataSelect==0 );
			m_selList.SetCheck( m_pConfig->m_iDataSelect==1 );
			m_selCurSel.SetCheck( m_pConfig->m_iDataSelect==2 );

			m_edtDeadtime.SetTextDouble( m_pConfig->m_dDeadTime );
			m_edtChannels.SetTextInt( m_pConfig->m_iChannels );
			m_edtDataChannel.SetTextInt( m_pConfig->m_iDataChannel );

			SetupStimulii(0);
			
			m_selModeEvaluate.SetCheck( m_pConfig->m_iMode!=OPT_MODE_OPTIMIZE );
			m_selModeOptimize.SetCheck( m_pConfig->m_iMode==OPT_MODE_OPTIMIZE );
			m_edtOptIter.SetTextInt( m_pConfig->m_iMaxIter );
			m_edtOptLLConv.SetTextDouble( m_pConfig->m_dConvLL );
			m_edtOptGradConv.SetTextDouble( m_pConfig->m_dConvGrad );
			m_selOptNumericDerivatives.SetCheck( m_pConfig->m_bNumericDerivatives );
			m_edtOptK0Step.SetTextDouble( m_pConfig->m_dK0Step );
			m_edtOptK1Step.SetTextDouble( m_pConfig->m_dK1Step );
			}
			
		//-----------------------------------------------------------------------------
		bool GetControlValues(  ) { 
			m_cmbDataSource.GetText( &(m_pConfig->m_cDataName) );
			m_pConfig->m_iDataSelect = m_selCurSel.GetCheck() ? 2 : m_selList.GetCheck() ? 1 : 0;
			m_pConfig->m_dDeadTime = m_edtDeadtime.GetTextDouble( );
			m_pConfig->m_iChannels = m_edtChannels.GetTextInt( );
			m_pConfig->m_iDataChannel = m_edtDataChannel.GetTextInt( );

			OnStimulusUpdate();

			m_pConfig->m_iMode = m_selModeEvaluate.GetCheck() ? OPT_MODE_EVALUATE : OPT_MODE_OPTIMIZE ;
			m_pConfig->m_iMaxIter = m_edtOptIter.GetTextInt( );
			m_pConfig->m_dConvLL = m_edtOptLLConv.GetTextDouble( );
			m_pConfig->m_dConvGrad = m_edtOptGradConv.GetTextDouble( );
			m_pConfig->m_bNumericDerivatives = m_selOptNumericDerivatives.GetCheck(  );
			m_pConfig->m_dK0Step = m_edtOptK0Step.GetTextDouble(  );
			m_pConfig->m_dK1Step = m_edtOptK1Step.GetTextDouble(  );
			return true;
			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [CANCEL]  
		void OnBtnCancel( ) { 
			EndDialog( m_hDlg, IDCANCEL );
			}
			
		//-- [SAVE] 
		void OnOk( ) { 
			if( ! GetControlValues() )
				return;
				
			//-- Selected data source 				
			int iData = m_pConfig->m_iDataSelect;		
			if( iData < 0 )	{
				MessageBox( GetParent(m_hDlg), "Select a data file to load from.", "Mil Rates", MB_OK );
				return;
				}
				
			//-- ptr to data window 
			int iWin = WindowListFindID( IDD_DATAVIEW, 0 );
			HWND hwndData;
			CDialog * pDataDlg;
			while( iWin != -1 ) { 
				hwndData = WindowListHandle( iWin );
				pDataDlg = (CDialog *) GetWindowLong( GetParent(hwndData), 0 );
				if( m_pConfig->m_cDataName==(char *)(pDataDlg->GetData( 1, NULL )) )
					break;
				iWin = WindowListFindID( IDD_DATAVIEW, iWin+1 );
				}
			if( iWin==-1 ) {
				MessageBox( GetParent(m_hDlg), "Selected data file is unavailable.", "Mil Rates", MB_OK );
				return;
				}

			//-- ptr to data 
			CData * pData = (CData *)(pDataDlg->GetData( 2, NULL ));
			int iChannels = pData->m_oConfig.m_aChannels.Count();

			//-- Data Channel
			if( m_pConfig->m_iDataChannel<0 || m_pConfig->m_iDataChannel>iChannels ) {		
				MessageBox(NULL, "Data Channel is out of range.", "Mil Rates", MB_OK );
				return;
				}
			
			//-- construct array m_pConfig->m_aiStimChan[m_iChannelStimulii] of stimuliii channels
			int iStimulii=0;
			for( int i=0; i<m_pConfig->m_aStim.Count(); ++i ) 
				if( ! m_pConfig->Stimulus(i)->m_bValue ) 
					++iStimulii;
			if( iStimulii>0 ) { 
				m_pConfig->m_oEventList.AllocateStimChan( iStimulii );	
				for( int i=0; i<m_pConfig->m_aStim.Count(); ++i ) 
					if( ! m_pConfig->Stimulus(i)->m_bValue ) { 
						m_pConfig->m_oEventList.AddStimChan( m_pConfig->Stimulus(i)->m_iChannel );
						}
				}						

			//-- Get list of data 
			int aiCurrent[4];					
			pDataDlg->GetData( 3, aiCurrent );
			int iList;							// get current selected list
			pDataDlg->GetData( 4, &iList );

			if( ! pData->GetEventList( m_pConfig->m_iDataChannel, 
						m_pConfig->m_iDataSelect, aiCurrent, iList, 						
						true/*joined*/, m_pConfig->m_dDeadTime, 
						&m_pConfig->m_oEventList, &m_pConfig->m_dSampling ) ) {
				MessageBox(NULL, "Data is empty or unavailable", "Mil Rates", MB_OK );
				return;
				}
		
			/*	//-- For debugging purposes - output the retrieved event list.
			#ifdef _DEBUG
			OutputDebugString( "GetEventList() ->" );
			char buf[100];
			for( int i=0; i<20 && i<m_pConfig->m_oEventList.m_iEvents; ++i ) {
				sprintf( buf, i%20==0 ? "\n{%d %d %d}" : "{%d %d %d} ", 
					int(m_pConfig->m_oEventList.m_aiClass[i]), 
					int(m_pConfig->m_oEventList.m_aiDuration[i]), 
					int(m_pConfig->m_oEventList.m_aiStimIndex[i]) );
				OutputDebugString( buf );
				}
			if( m_pConfig->m_oEventList.m_iEvents>=20 )
				OutputDebugString( " ... ");
			OutputDebugString( "\n--------------------\n");
			#endif
			*/

			if( m_pConfig->m_oEventList.m_iEvents==0 ) {
				MessageBox(NULL, "Data is empty.", "Mil Rates", MB_OK );
				return;
				}

			EndDialog( m_hDlg, IDOK );
			}
			
	};

//-----------------------------------------------------------------------------
CVwMil * g_pVwMil;		// Modal - one and only instance

//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK VwMilDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId, wmEvent;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwMil->m_hDlg=hDlg;			

			//-- Clear Existing  Values
			g_pVwMil->InitializeControls();
			g_pVwMil->SetControlValues();
			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			switch( wmId ) { 
				case IDOK :					//-- [OK]
					g_pVwMil->OnOk( );
					break;

				case IDCANCEL :				//-- [CANCEL]
					g_pVwMil->OnBtnCancel( );
					break;
					
				case IDC_LST_STIMULII : 
					if( wmEvent == LBN_SELCHANGE )
						g_pVwMil->OnStimulusSelectionChanged( );
					break;
				
				case IDC_OPTSTIMTYPECHANNEL : 
				case IDC_OPTSTIMTYPEVALUE :
					g_pVwMil->OnStimulusUpdate();
					break;
				
				case IDC_EDT_STIMVALUE : 
				case IDC_EDT_STIMCHANNEL : 
					if( wmEvent==EN_CHANGE )
						g_pVwMil->OnStimulusUpdate();
					break;
				}
				
			return TRUE;

		case WM_PAINT : 
			/*{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hDlg, &ps);

			EndPaint(hDlg, &ps);
			}*/
			return false;

		case WM_DESTROY : 
			delete g_pVwMil;
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 

//-----------------------------------------------------------------------------
void OpenVwMil( HWND hDlg, CModel * pModel ) { 
	//-- create structure to pass option to / from dialogue.
	// Note : pConfig->m_oModel is not set to pModel, it is only used for task execute.
	COptionsMil * pConfig = new COptionsMil;
	
	pConfig->RegistryLoad();

	//-- Get array of model stimulus names, then merge with loaded parameters
	CStringArray aNames;
	pModel->GetStimuliiNames( aNames );
	int iName;
	COptionsMilStimulus * pStim;

	for( int iStim=pConfig->m_aStim.Count()-1; iStim>=0; --iStim ) { 
		pStim = pConfig->Stimulus( iStim );
		iName = aNames.Find( pStim->m_cName.ConstBuffer() );
		if( iName>=0 )
			aNames.DeleteString( iName );			// remove model parameters already config'd in prior-run.
		else 
			pConfig->m_aStim.DeleteItem( iStim );	// remove prior-run stim list items not in model.
		}
			
	//-- for model parameters not in registry list, set up as new parameter item 
	for( iName=0; iName<aNames.Count(); ++iName )
		pConfig->m_aStim.AddItem( new COptionsMilStimulus(aNames[iName].ConstBuffer(), 0, 0.0, true ) );

	//-- create dialogue to get task options 	
	g_pVwMil = new CVwMil;
	g_pVwMil->m_pConfig=pConfig;
	
	//-- present dialogue, execute if [OK]
	if( DialogBox( GetInstance(), (LPCTSTR)IDD_TASK_MIL, hDlg, (DLGPROC)VwMilDlgProc ) == IDOK ) { 
		pConfig->RegistrySave();
		
		//-- Open the task & progress view 	
		CTaskMil * pTaskMil = new CTaskMil( );
		pTaskMil->m_pOptionsMil = pConfig;		// transfer ownership to task (for later delete)
		pTaskMil->m_pOptionsMil->m_oModel = *pModel;
		pTaskMil->m_pOptionsMil->m_pOriginalModel = pModel;
		pTaskMil->m_hModelWnd = hDlg;

		//-- Open Task thread 		
		HWND hVwTask = pTaskMil->CreateTaskWindow( GetClientWin(), GetChildClassName() );
		pTaskMil->m_hThread = CreateThread( NULL, 0, TaskMil, pTaskMil, 0, NULL );
		} 
	else 
		delete pConfig;
		
	}

